@@ -117,7 +117,7 @@ KGoldrunner::KGoldrunner()
     // Get catalog for translation.
     KGlobal::locale()->insertCatalog ( QLatin1String( "libkdegames" ));
 
-    // Tell the KMainWindow that the KGrCanvas object is the main widget.
+    // Tell the KMainWindow that the KGrView object is the main widget.
     setCentralWidget (view);
 
     scene       = view->gameScene ();
@@ -165,6 +165,9 @@ KGoldrunner::KGoldrunner()
     // Besides, we cannot render it until after the initial resize event (s).
     toolBar ("editToolbar")->hide();
 
+    // This is needed to make the arrow keys control the hero properly.
+    setUpKeyboardControl();
+
     // Do NOT paint main widget yet (title bar, menu, blank playfield).
     // Instead, queue a call to the "KGoldrunner_2" constructor extension.
     QMetaObject::invokeMethod (this, "KGoldrunner_2", Qt::QueuedConnection);
@@ -653,13 +656,13 @@ void KGoldrunner::keyControl (const QString & name, const QString & text,
     KAction * a = actionCollection()->addAction (name);
     a->setText (text);
     a->setShortcut (shortcut);
+    a->setAutoRepeat (false);		// Avoid repeats of signals by QAction.
 
     // If this is a move-key, let keyPressEvent() through, instead of signal.
     if (mover) {
         a->setEnabled (false);
-    }
-    else {
-        a->setAutoRepeat (false);	// Else, prevent QAction signal repeat.
+	addAction (a);
+	return;
     }
 
     connect (a, SIGNAL (triggered(bool)), tempMapper, SLOT (map()));
@@ -667,6 +670,20 @@ void KGoldrunner::keyControl (const QString & name, const QString & text,
     addAction (a);
 }
 
+void KGoldrunner::setUpKeyboardControl()
+{
+    // This is needed to ensure that the press and release of Up, Down, Left and
+    // Right keys (arrow-keys) are all received as required.
+    //
+    // If the KGoldrunner widget does not have the keyboard focus, arrow-keys
+    // provide only key-release events, which do not control the hero properly.
+    // Other keys provide both press and release events, regardless of focus.
+
+    this->setFocusPolicy (Qt::StrongFocus); // Tab or click gets the focus.
+    view->setFocusProxy (this);		    // So does a click on the play-area.
+    this->setFocus (Qt::OtherFocusReason);  // And we start by having the focus.
+}
+
 void KGoldrunner::keyPressEvent (QKeyEvent * event)
 {
     // For movement keys, all presses and releases are processed, thus allowing
@@ -686,28 +703,35 @@ void KGoldrunner::keyReleaseEvent (QKeyEvent * event)
 
 bool KGoldrunner::identifyMoveAction (QKeyEvent * event, bool pressed)
 {
-    bool result = false;
-    if (! event->isAutoRepeat()) {
-        QKeySequence keystroke (event->key() + event->modifiers());
-        result = true;
-
-        if ((ACTION ("move_left"))->shortcuts().contains(keystroke)) {
-            game->kbControl (LEFT, pressed);
-        }
-        else if ((ACTION ("move_right"))->shortcuts().contains(keystroke)) {
-            game->kbControl (RIGHT, pressed);
-        }
-        else if ((ACTION ("move_up"))->shortcuts().contains(keystroke)) {
-            game->kbControl (UP, pressed);
-        }
-        else if ((ACTION ("move_down"))->shortcuts().contains(keystroke)) {
-            game->kbControl (DOWN, pressed);
-        }
-        else {
-            result = false;
-        }
+    if (event->isAutoRepeat()) {
+        return false;		// Use only the release and the initial press.
     }
-    return result;
+    Direction dirn = STAND;
+    // The arrow keys show the "Keypad" modifier as being set, even if the
+    // computer does NOT have a keypad (see Qt::KeypadModifier doco). It is
+    // OK to ignore the Keypad modifier (see the code for "qevent.cpp" at
+    // "bool QKeyEvent::matches(QKeySequence::StandardKey matchKey)"). The
+    // keys on the keypad usually have equivalents on the main keyboard.
+    QKeySequence keystroke (~(Qt::KeypadModifier) &
+                             (event->key() | event->modifiers()));
+    if ((ACTION ("move_left"))->shortcuts().contains(keystroke)) {
+        dirn = LEFT;
+    }
+    else if ((ACTION ("move_right"))->shortcuts().contains(keystroke)) {
+        dirn = RIGHT;
+    }
+    else if ((ACTION ("move_up"))->shortcuts().contains(keystroke)) {
+        dirn = UP;
+    }
+    else if ((ACTION ("move_down"))->shortcuts().contains(keystroke)) {
+        dirn = DOWN;
+    }
+    else {
+        return false;
+    }
+    // Use this event to control the hero, if KEYBOARD mode is selected.
+    game->kbControl (dirn, pressed);
+    return true;
 }
 
 void KGoldrunner::viewFullScreen (bool activation)
